'use strict';

var system = require('@solana-program/system');
var addressLookupTable = require('@solana-program/address-lookup-table');
var computeBudget = require('@solana-program/compute-budget');
var instructions = require('@solana/instructions');
var codecs = require('@solana/codecs');
var addresses = require('@solana/addresses');
var signers = require('@solana/signers');

// src/programs/index.ts
function isSetComputeLimitInstruction(instruction) {
  return instructions.isInstructionForProgram(instruction, computeBudget.COMPUTE_BUDGET_PROGRAM_ADDRESS) && instructions.isInstructionWithData(instruction) && instruction.data[0] === computeBudget.ComputeBudgetInstruction.SetComputeUnitLimit;
}
function hasSetComputeLimitInstruction(tx) {
  return tx.instructions.filter(isSetComputeLimitInstruction).length == 1;
}
function isSetComputeUnitPriceInstruction(instruction) {
  return instructions.isInstructionForProgram(instruction, computeBudget.COMPUTE_BUDGET_PROGRAM_ADDRESS) && instructions.isInstructionWithData(instruction) && instruction.data[0] === computeBudget.ComputeBudgetInstruction.SetComputeUnitPrice;
}
function hasSetComputeUnitPriceInstruction(tx) {
  return tx.instructions.filter(isSetComputeUnitPriceInstruction).length == 1;
}

// src/programs/memo/generated/programs/memo.ts
var MEMO_PROGRAM_ADDRESS = "MemoSq4gqABAXKb96qnH8TysNcWxMyWCqXgDLGmfcHr";
var MemoInstruction = /* @__PURE__ */ ((MemoInstruction2) => {
  MemoInstruction2[MemoInstruction2["AddMemo"] = 0] = "AddMemo";
  return MemoInstruction2;
})(MemoInstruction || {});

// src/programs/memo/generated/instructions/addMemo.ts
function getAddMemoInstructionDataEncoder() {
  return codecs.getStructEncoder([["memo", codecs.getUtf8Encoder()]]);
}
function getAddMemoInstructionDataDecoder() {
  return codecs.getStructDecoder([["memo", codecs.getUtf8Decoder()]]);
}
function getAddMemoInstructionDataCodec() {
  return codecs.combineCodec(getAddMemoInstructionDataEncoder(), getAddMemoInstructionDataDecoder());
}
function getAddMemoInstruction(input, config) {
  const programAddress = config?.programAddress ?? MEMO_PROGRAM_ADDRESS;
  const args = { ...input };
  const remainingAccounts = (args.signers ?? []).map((signer) => ({
    address: signer.address,
    role: instructions.AccountRole.READONLY_SIGNER,
    signer
  }));
  const instruction = {
    accounts: remainingAccounts,
    programAddress,
    data: getAddMemoInstructionDataEncoder().encode(args)
  };
  return instruction;
}
function parseAddMemoInstruction(instruction) {
  return {
    programAddress: instruction.programAddress,
    data: getAddMemoInstructionDataDecoder().decode(instruction.data)
  };
}
function expectAddress(value) {
  if (!value) {
    throw new Error("Expected a Address.");
  }
  if (typeof value === "object" && "address" in value) {
    return value.address;
  }
  if (Array.isArray(value)) {
    return value[0];
  }
  return value;
}
function getAccountMetaFactory(programAddress, optionalAccountStrategy) {
  return (account) => {
    if (!account.value) {
      return;
    }
    const writableRole = account.isWritable ? instructions.AccountRole.WRITABLE : instructions.AccountRole.READONLY;
    return Object.freeze({
      address: expectAddress(account.value),
      role: isTransactionSigner(account.value) ? instructions.upgradeRoleToSigner(writableRole) : writableRole,
      ...isTransactionSigner(account.value) ? { signer: account.value } : {}
    });
  };
}
function isTransactionSigner(value) {
  return !!value && typeof value === "object" && "address" in value && signers.isTransactionSigner(value);
}
function getCollectionEncoder() {
  return codecs.getStructEncoder([
    ["verified", codecs.getBooleanEncoder()],
    ["key", addresses.getAddressEncoder()]
  ]);
}
function getCollectionDecoder() {
  return codecs.getStructDecoder([
    ["verified", codecs.getBooleanDecoder()],
    ["key", addresses.getAddressDecoder()]
  ]);
}
function getCollectionCodec() {
  return codecs.combineCodec(getCollectionEncoder(), getCollectionDecoder());
}
function getCollectionDetailsEncoder() {
  return codecs.getDiscriminatedUnionEncoder([
    ["V1", codecs.getStructEncoder([["size", codecs.getU64Encoder()]])],
    ["V2", codecs.getStructEncoder([["padding", codecs.fixEncoderSize(codecs.getBytesEncoder(), 8)]])]
  ]);
}
function getCollectionDetailsDecoder() {
  return codecs.getDiscriminatedUnionDecoder([
    ["V1", codecs.getStructDecoder([["size", codecs.getU64Decoder()]])],
    ["V2", codecs.getStructDecoder([["padding", codecs.fixDecoderSize(codecs.getBytesDecoder(), 8)]])]
  ]);
}
function getCollectionDetailsCodec() {
  return codecs.combineCodec(getCollectionDetailsEncoder(), getCollectionDetailsDecoder());
}
function collectionDetails(kind, data) {
  return Array.isArray(data) ? { __kind: kind, fields: data } : { __kind: kind, ...data ?? {} };
}
function isCollectionDetails(kind, value) {
  return value.__kind === kind;
}
function getCollectionDetailsToggleEncoder() {
  return codecs.getDiscriminatedUnionEncoder([
    ["None", codecs.getUnitEncoder()],
    ["Clear", codecs.getUnitEncoder()],
    ["Set", codecs.getStructEncoder([["fields", codecs.getTupleEncoder([getCollectionDetailsEncoder()])]])]
  ]);
}
function getCollectionDetailsToggleDecoder() {
  return codecs.getDiscriminatedUnionDecoder([
    ["None", codecs.getUnitDecoder()],
    ["Clear", codecs.getUnitDecoder()],
    ["Set", codecs.getStructDecoder([["fields", codecs.getTupleDecoder([getCollectionDetailsDecoder()])]])]
  ]);
}
function getCollectionDetailsToggleCodec() {
  return codecs.combineCodec(getCollectionDetailsToggleEncoder(), getCollectionDetailsToggleDecoder());
}
function collectionDetailsToggle(kind, data) {
  return Array.isArray(data) ? { __kind: kind, fields: data } : { __kind: kind, ...data ?? {} };
}
function isCollectionDetailsToggle(kind, value) {
  return value.__kind === kind;
}
function getCreatorEncoder() {
  return codecs.getStructEncoder([
    ["address", addresses.getAddressEncoder()],
    ["verified", codecs.getBooleanEncoder()],
    ["share", codecs.getU8Encoder()]
  ]);
}
function getCreatorDecoder() {
  return codecs.getStructDecoder([
    ["address", addresses.getAddressDecoder()],
    ["verified", codecs.getBooleanDecoder()],
    ["share", codecs.getU8Decoder()]
  ]);
}
function getCreatorCodec() {
  return codecs.combineCodec(getCreatorEncoder(), getCreatorDecoder());
}
function getDataV2Encoder() {
  return codecs.getStructEncoder([
    ["name", codecs.addEncoderSizePrefix(codecs.getUtf8Encoder(), codecs.getU32Encoder())],
    ["symbol", codecs.addEncoderSizePrefix(codecs.getUtf8Encoder(), codecs.getU32Encoder())],
    ["uri", codecs.addEncoderSizePrefix(codecs.getUtf8Encoder(), codecs.getU32Encoder())],
    ["sellerFeeBasisPoints", codecs.getU16Encoder()],
    ["creators", codecs.getOptionEncoder(codecs.getArrayEncoder(getCreatorEncoder()))],
    ["collection", codecs.getOptionEncoder(getCollectionEncoder())],
    ["uses", codecs.getOptionEncoder(getUsesEncoder())]
  ]);
}
function getDataV2Decoder() {
  return codecs.getStructDecoder([
    ["name", codecs.addDecoderSizePrefix(codecs.getUtf8Decoder(), codecs.getU32Decoder())],
    ["symbol", codecs.addDecoderSizePrefix(codecs.getUtf8Decoder(), codecs.getU32Decoder())],
    ["uri", codecs.addDecoderSizePrefix(codecs.getUtf8Decoder(), codecs.getU32Decoder())],
    ["sellerFeeBasisPoints", codecs.getU16Decoder()],
    ["creators", codecs.getOptionDecoder(codecs.getArrayDecoder(getCreatorDecoder()))],
    ["collection", codecs.getOptionDecoder(getCollectionDecoder())],
    ["uses", codecs.getOptionDecoder(getUsesDecoder())]
  ]);
}
function getDataV2Codec() {
  return codecs.combineCodec(getDataV2Encoder(), getDataV2Decoder());
}
function getUsesEncoder() {
  return codecs.getStructEncoder([
    ["useMethod", getUseMethodEncoder()],
    ["remaining", codecs.getU64Encoder()],
    ["total", codecs.getU64Encoder()]
  ]);
}
function getUsesDecoder() {
  return codecs.getStructDecoder([
    ["useMethod", getUseMethodDecoder()],
    ["remaining", codecs.getU64Decoder()],
    ["total", codecs.getU64Decoder()]
  ]);
}
function getUsesCodec() {
  return codecs.combineCodec(getUsesEncoder(), getUsesDecoder());
}
var UseMethod = /* @__PURE__ */ ((UseMethod2) => {
  UseMethod2[UseMethod2["Burn"] = 0] = "Burn";
  UseMethod2[UseMethod2["Multiple"] = 1] = "Multiple";
  UseMethod2[UseMethod2["Single"] = 2] = "Single";
  return UseMethod2;
})(UseMethod || {});
function getUseMethodEncoder() {
  return codecs.getEnumEncoder(UseMethod);
}
function getUseMethodDecoder() {
  return codecs.getEnumDecoder(UseMethod);
}
function getUseMethodCodec() {
  return codecs.combineCodec(getUseMethodEncoder(), getUseMethodDecoder());
}

// src/programs/token-metadata/generated/instructions/createMetadataAccountV3.ts
var CREATE_METADATA_ACCOUNT_V3_DISCRIMINATOR = 33;
var TOKEN_METADATA_PROGRAM_ADDRESS = "metaqbxxUerdq28cj1RbAWkYQm3ybzjb6a8bt518x1s";
function getCreateMetadataAccountV3DiscriminatorBytes() {
  return codecs.getU8Encoder().encode(CREATE_METADATA_ACCOUNT_V3_DISCRIMINATOR);
}
function getCreateMetadataAccountV3InstructionDataEncoder() {
  return codecs.transformEncoder(
    codecs.getStructEncoder([
      ["discriminator", codecs.getU8Encoder()],
      ["data", getDataV2Encoder()],
      ["isMutable", codecs.getBooleanEncoder()],
      ["collectionDetails", codecs.getOptionEncoder(getCollectionDetailsEncoder())]
    ]),
    (value) => ({
      ...value,
      discriminator: CREATE_METADATA_ACCOUNT_V3_DISCRIMINATOR
    })
  );
}
function getCreateMetadataAccountV3InstructionDataDecoder() {
  return codecs.getStructDecoder([
    ["discriminator", codecs.getU8Decoder()],
    ["data", getDataV2Decoder()],
    ["isMutable", codecs.getBooleanDecoder()],
    ["collectionDetails", codecs.getOptionDecoder(getCollectionDetailsDecoder())]
  ]);
}
function getCreateMetadataAccountV3InstructionDataCodec() {
  return codecs.combineCodec(
    getCreateMetadataAccountV3InstructionDataEncoder(),
    getCreateMetadataAccountV3InstructionDataDecoder()
  );
}
function getCreateMetadataAccountV3Instruction(input, config) {
  const programAddress = config?.programAddress ?? TOKEN_METADATA_PROGRAM_ADDRESS;
  const originalAccounts = {
    metadata: { value: input.metadata ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: false },
    mintAuthority: { value: input.mintAuthority ?? null, isWritable: false },
    payer: { value: input.payer ?? null, isWritable: true },
    updateAuthority: {
      value: input.updateAuthority ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    rent: { value: input.rent ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const getAccountMeta = getAccountMetaFactory();
  const instruction = {
    accounts: [
      getAccountMeta(accounts.metadata),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.mintAuthority),
      getAccountMeta(accounts.payer),
      getAccountMeta(accounts.updateAuthority),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.rent)
    ].filter((x) => x !== void 0),
    programAddress,
    data: getCreateMetadataAccountV3InstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseCreateMetadataAccountV3Instruction(instruction) {
  if (instruction.accounts.length < 6) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  let optionalAccountsRemaining = instruction.accounts.length - 6;
  const getNextOptionalAccount = () => {
    if (optionalAccountsRemaining === 0) return void 0;
    optionalAccountsRemaining -= 1;
    return getNextAccount();
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      metadata: getNextAccount(),
      mint: getNextAccount(),
      mintAuthority: getNextAccount(),
      payer: getNextAccount(),
      updateAuthority: getNextAccount(),
      systemProgram: getNextAccount(),
      rent: getNextOptionalAccount()
    },
    data: getCreateMetadataAccountV3InstructionDataDecoder().decode(instruction.data)
  };
}

// src/core/utils.ts
function checkedAddress(input) {
  return typeof input == "string" ? input : input.address;
}

// src/programs/token-metadata/addresses.ts
async function getTokenMetadataAddress(mint) {
  return (await addresses.getProgramDerivedAddress({
    programAddress: TOKEN_METADATA_PROGRAM_ADDRESS,
    seeds: [
      Buffer.from("metadata"),
      addresses.getAddressEncoder().encode(TOKEN_METADATA_PROGRAM_ADDRESS),
      addresses.getAddressEncoder().encode(checkedAddress(mint))
    ]
  }))[0];
}

exports.CREATE_METADATA_ACCOUNT_V3_DISCRIMINATOR = CREATE_METADATA_ACCOUNT_V3_DISCRIMINATOR;
exports.MEMO_PROGRAM_ADDRESS = MEMO_PROGRAM_ADDRESS;
exports.MemoInstruction = MemoInstruction;
exports.TOKEN_METADATA_PROGRAM_ADDRESS = TOKEN_METADATA_PROGRAM_ADDRESS;
exports.UseMethod = UseMethod;
exports.collectionDetails = collectionDetails;
exports.collectionDetailsToggle = collectionDetailsToggle;
exports.getAddMemoInstruction = getAddMemoInstruction;
exports.getAddMemoInstructionDataCodec = getAddMemoInstructionDataCodec;
exports.getAddMemoInstructionDataDecoder = getAddMemoInstructionDataDecoder;
exports.getAddMemoInstructionDataEncoder = getAddMemoInstructionDataEncoder;
exports.getCollectionCodec = getCollectionCodec;
exports.getCollectionDecoder = getCollectionDecoder;
exports.getCollectionDetailsCodec = getCollectionDetailsCodec;
exports.getCollectionDetailsDecoder = getCollectionDetailsDecoder;
exports.getCollectionDetailsEncoder = getCollectionDetailsEncoder;
exports.getCollectionDetailsToggleCodec = getCollectionDetailsToggleCodec;
exports.getCollectionDetailsToggleDecoder = getCollectionDetailsToggleDecoder;
exports.getCollectionDetailsToggleEncoder = getCollectionDetailsToggleEncoder;
exports.getCollectionEncoder = getCollectionEncoder;
exports.getCreateMetadataAccountV3DiscriminatorBytes = getCreateMetadataAccountV3DiscriminatorBytes;
exports.getCreateMetadataAccountV3Instruction = getCreateMetadataAccountV3Instruction;
exports.getCreateMetadataAccountV3InstructionDataCodec = getCreateMetadataAccountV3InstructionDataCodec;
exports.getCreateMetadataAccountV3InstructionDataDecoder = getCreateMetadataAccountV3InstructionDataDecoder;
exports.getCreateMetadataAccountV3InstructionDataEncoder = getCreateMetadataAccountV3InstructionDataEncoder;
exports.getCreatorCodec = getCreatorCodec;
exports.getCreatorDecoder = getCreatorDecoder;
exports.getCreatorEncoder = getCreatorEncoder;
exports.getDataV2Codec = getDataV2Codec;
exports.getDataV2Decoder = getDataV2Decoder;
exports.getDataV2Encoder = getDataV2Encoder;
exports.getTokenMetadataAddress = getTokenMetadataAddress;
exports.getUseMethodCodec = getUseMethodCodec;
exports.getUseMethodDecoder = getUseMethodDecoder;
exports.getUseMethodEncoder = getUseMethodEncoder;
exports.getUsesCodec = getUsesCodec;
exports.getUsesDecoder = getUsesDecoder;
exports.getUsesEncoder = getUsesEncoder;
exports.hasSetComputeLimitInstruction = hasSetComputeLimitInstruction;
exports.hasSetComputeUnitPriceInstruction = hasSetComputeUnitPriceInstruction;
exports.isCollectionDetails = isCollectionDetails;
exports.isCollectionDetailsToggle = isCollectionDetailsToggle;
exports.isSetComputeLimitInstruction = isSetComputeLimitInstruction;
exports.isSetComputeUnitPriceInstruction = isSetComputeUnitPriceInstruction;
exports.parseAddMemoInstruction = parseAddMemoInstruction;
exports.parseCreateMetadataAccountV3Instruction = parseCreateMetadataAccountV3Instruction;
Object.keys(system).forEach(function (k) {
  if (k !== 'default' && !Object.prototype.hasOwnProperty.call(exports, k)) Object.defineProperty(exports, k, {
    enumerable: true,
    get: function () { return system[k]; }
  });
});
Object.keys(addressLookupTable).forEach(function (k) {
  if (k !== 'default' && !Object.prototype.hasOwnProperty.call(exports, k)) Object.defineProperty(exports, k, {
    enumerable: true,
    get: function () { return addressLookupTable[k]; }
  });
});
Object.keys(computeBudget).forEach(function (k) {
  if (k !== 'default' && !Object.prototype.hasOwnProperty.call(exports, k)) Object.defineProperty(exports, k, {
    enumerable: true,
    get: function () { return computeBudget[k]; }
  });
});
//# sourceMappingURL=index.node.cjs.map
//# sourceMappingURL=index.node.cjs.map